<?php
// Copyright (c) 2005-2009, RedIRIS. All Rights Reserved.
//
// You may distribute under the terms of the GNU General Public License,
// as specified in the LICENSE file that was shipped with this distribution

include("Authz.php");

class AuthzDBA implements Authz {

	var $cfg;
	var $error;

    public function configure($file,$section) {
    	$this->cfg['DB_Type'] = "db4";
    	$this->cfg['Invite_Text_es'] = "Ha recibido una invitacion para acceder a ".$section.":\n\n";
    	$this->cfg['Invite_Text_en'] = "You have received an invitation to access to ".$section.":\n\n";
    	$this->cfg['Invite_Title_es'] = "Invitacion para unirse a ".$section;
    	$this->cfg['Invite_Title_en'] = "Please join ".$section;

    	$c = parse_ini_file($file,true);
    	$params = Array (
			"DB_File",
			"Invite_DB_File",
			"Authz_Attribute"
		);
		
		foreach ($c['Authz_Main'] as $key => $value) {
			$this->cfg[$key] = $value;
		}
		foreach ($c[$section] as $key => $value) {
			$this->cfg[$key] = $value;
		}
		foreach ($params as $param) {
			if (!isset($this->cfg[$param])) {
				$this->error = "AuthzDBA::configure(): required parameter $param does not exist.";
			}
		}
    }

	public function getError() {
		return $this->error;
	}

    public function isAuthorized($user, $attrs) {
    	$id = dba_open($this->cfg["DB_File"], "r", $this->cfg['DB_Type']);
		if (!$id) {
			$this->error = "AuthzDBA::isAuthorized(): cannot open database file ".$this->cfg['DB_File'].".";
			return false;
		} elseif (!(dba_exists($user, $id))) {
			$this->error = "AuthzDBA::isAuthorized(): key $user does not exist in database ".$this->cfg['DB_File'].".";
			dba_close($id);
			return false;
		}

		$value = dba_fetch($user, $id);
		if ($value != $attrs[$this->cfg['Authz_Attribute']]) {
			$this->error = "AuthzDbA::isAuthorized(): user ".$this->cfg['Authz_Attribute']." does not match. Expected '".
							$value."', '".$attrs[$this->cfg['Authz_Attribute']]."' found.";
			return false;
		}
		dba_close($id);
		
        return true;
    }

    public function authorize($user,$attrs,$ref,$force = false) {
    	$idb = dba_open($this->cfg['Invite_DB_File'], "w", $this->cfg['DB_Type']);
    	if (!$idb && !$force) {
    		$this->error = "AuthzDBA::authorize(): cannot open database file ".$this->cfg['Invite_DB_File'].".";
			return false;
    	}

    	$adb = dba_open($this->cfg["DB_File"], "c", $this->cfg['DB_Type']);
		if (!$adb) {
			$this->error = "AuthzDBA::authorize(): cannot open database file ".$this->cfg['DB_File'].".";
			return false;
		}

		if (!$force && !(dba_exists($ref, $idb))) {
			if (dba_exists($user, $adb)) {
				$attr = dba_fetch($user, $adb);
				if ($attr = $attrs[$this->cfg['Authz_Attribute']]) {
					dba_close($idb);
					dba_close($adb);
					return true;
				}
			} 
    		$this->error = "AuthzDBA::authorize(): no such invite $ref registered.";
			dba_close($idb);
			return false;
    	}

		dba_replace($user, $attrs[$this->cfg['Authz_Attribute']], $adb);
		dba_delete($ref,$idb);
		dba_close($idb);
		dba_close($adb);
		return true;
    }

	public function invite($mail,$lang,$url) {
		$ref = mt_rand();
		$id = dba_open($this->cfg['Invite_DB_File'], "c", $this->cfg['DB_Type']);
		if (!$id) {
			$this->error = "AuthzDBA::invite(): cannot open database file ".$this->cfg['Invite_DB_File'].".";
			return false;
		}
		dba_replace($ref, $mail, $id);
		dba_close($id);
		$sep = (!strstr($url,"?")) ? "?" : "&";
		$text = $this->cfg['Invite_Text_'.$lang].$url.$sep."ref=".$ref;
		$subject = $this->cfg['Invite_Title'][$lang];
		$headers = (isset($this->cfg['Admin_Email'])) ? "From: ".$this->cfg['Admin_Email'] : "";
		mail($mail,$subject,$text,$headers);
		return true;
	}

}

?>